package kz.arta.synergy.workflow.mobile.android.common.utils;

import android.app.Activity;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.util.DisplayMetrics;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.MimeTypeMap;
import android.widget.GridView;
import android.widget.ListAdapter;
import android.widget.ListView;

import com.devspark.appmsg.AppMsg;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import kz.arta.synergy.workflow.mobile.android.R;
import kz.arta.synergy.workflow.mobile.android.common.SettingsConst;
import kz.arta.synergy.workflow.mobile.android.common.models.Person;
import kz.arta.synergy.workflow.mobile.android.common.models.settings.CompletionFormItem;
import kz.arta.synergy.workflow.mobile.android.common.models.settings.ProcessItem;
import kz.arta.synergy.workflow.mobile.android.documents.resolution.models.ItemResolution;
import kz.arta.synergy.workflow.mobile.android.documents.resolution.models.ResolutionWorkType;
import kz.arta.synergy.workflow.mobile.android.documents.route.RoutesConst;

/**
 * Created by neox on 27.02.15.
 * Общие утилиты приложения
 */
public class Utils {

    public static void setListViewHeightBasedOnChildren(ListView listView) {
        ListAdapter listAdapter = listView.getAdapter();
        if (listAdapter == null) {
            // pre-condition
            return;
        }
        int totalHeight = listView.getPaddingTop() + listView.getPaddingBottom();
        for (int i = 0; i < listAdapter.getCount(); i++) {
            View listItem = listAdapter.getView(i, null, listView);
            if (listItem instanceof ViewGroup) {
                listItem.setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT));
            }
            listItem.measure(0, 0);
            totalHeight += listItem.getMeasuredHeight();
        }

        ViewGroup.LayoutParams params = listView.getLayoutParams();
        params.height = totalHeight + (listView.getDividerHeight() * (listAdapter.getCount() - 1));
        listView.setLayoutParams(params);
    }

    public static void copyStream(InputStream is, OutputStream os) {
        final int buffer_size = 1024;
        try {
            byte[] bytes = new byte[buffer_size];
            for (; ; ) {
                int count = is.read(bytes, 0, buffer_size);
                if (count == -1)
                    break;
                os.write(bytes, 0, count);
            }
        } catch (Exception ex) {
            Logger.v(ex.getLocalizedMessage());
        }
    }

    /**
     * Метод конвертирует dp в px
     *
     * @param dp - dp
     * @return px
     */
    public static int convertDpToPixel(float dp, Context context) {
        DisplayMetrics metrics = context.getResources().getDisplayMetrics();
        float px = dp * (metrics.densityDpi / 160f);
        return (int) px;
    }

    /**
     * Проверка включен ли интернет
     */
    public static boolean isOnline(Context context, String host) {
        ConnectivityManager cm = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo netInfo = cm.getActiveNetworkInfo();
        return netInfo != null && netInfo.isConnected();
    }

    /**
     * Метод из имени файла получает mime-type
     *
     * @param filename - имя файла
     * @return - mime-type
     */
    public static String getMimeByFilename(String filename) {
        String ext;
        String type;
        int lastDot = filename.lastIndexOf(".");
        if (lastDot > 0) {
            ext = filename.substring(lastDot + 1);
            MimeTypeMap mime = MimeTypeMap.getSingleton();
            type = mime.getMimeTypeFromExtension(ext);
            if (type != null) {
                return type;
            }
        }
        return "application/octet-stream";
    }

    /**
     * Включена ли синхронизация
     */
    public static boolean checkSyncEnables(SharedPreferences settings) {
        return settings.contains(SettingsConst.SYNC_CHECKBOX) && settings.getBoolean(SettingsConst.SYNC_CHECKBOX, false);
    }

    /**
     * Метод возвращает форму завершения по ее идентификатору, если она есть
     */
    public static String getFormCompletion(String form, Person person, Context context, boolean resolution) {
        if (form == null && person == null) {
            return "";
        }
        for (CompletionFormItem item : person.getSettings().getWorkCompletionForms()) {
            if (item.getId().equals(form)) {
                return item.getName();
            }
        }
        return context.getString((resolution ? R.string.work_result : R.string.no));
    }

    /**
     * Метод возвращает тип процесса для резолюций по идентиификатору
     */
    public static String getProcessByDictId(String dictID, Person person) {
        if (person.getSettings().getResolutionWorkType() == null || person.getSettings().getResolutionWorkType().length == 0) {
            return "";
        }
        for (ResolutionWorkType resolutionWorkType : person.getSettings().getResolutionWorkType()) {
            if (resolutionWorkType.getDictID().equals(dictID)) {
                return resolutionWorkType.getItemProcess();
            }
        }
        return "";
    }

    /**
     * Метод возвращает пользователя
     */
    public static String parseUser(String user) {
        if (user != null && user.length() != 0) {
            return parseAuthor(user);
        } else {
            return "";
        }
    }

    /**
     * Метод возвращает строку, в которой пользователи перечислены через запятую, если они есть
     */
    public static String parseExecutors(ItemResolution.ResolutionsUsers[] users) {
        if (users == null) {
            return "";
        } else {
            if (users.length != 0) {
                StringBuilder usersStr = new StringBuilder();
                for (ItemResolution.ResolutionsUsers user : users) {
                    usersStr.append(parseAuthor(user.getUser())).append(", ");
                }
                return usersStr.toString().substring(0, usersStr.length() - 2);
            } else {
                return "";
            }
        }
    }

    /**
     * Возвращает формат файла зная его путь
     */
    public static String getFormatByFilename(String result) {
        return result.substring(result.lastIndexOf("."), result.length());
    }

    /**
     * Добавляет первый элемент для форм заврешения (сервер не возвращает его, но для диалогов он нужен)
     */
    public static String[] parseFormCompletion(String[] items, String text) {
        List<String> list = new ArrayList<>();
        list.add(text);
        Collections.addAll(list, items);
        return list.toArray(new String[list.size()]);
    }

    /**
     * Парсинг url'а до иконки файла
     */
    public static String parseUrlToIcon(String url) {
        try {
            if (url.contains("fileName=")) {
                int index = url.lastIndexOf("fileName=") + 9;
                String replaceStr = url.substring(index, url.length());
                replaceStr = URLEncoder.encode(replaceStr, "utf-8");
                StringBuilder result = new StringBuilder(url);
                result.replace(index, result.length(), replaceStr);
                return result.toString();
            } else {
                return url;
            }
        } catch (Exception exc) {
            return url;
        }
    }

    /**
     * Изменяет имя пользователя из полного (Бобров Степан Григорьевич) в сокращенное (Бобров С. Г)
     *
     * @param fullAuthorName полное имя
     * @return сокращенное имя
     */
    public static String parseAuthor(String fullAuthorName) {
        StringBuilder result = new StringBuilder();
        String[] arr = fullAuthorName.split(" ");
        if (arr.length == 3) {
            result.append(arr[0]).append(" ");
            result.append(arr[1].substring(0, 1)).append(". ");
            result.append(arr[2].substring(0, 1)).append(". ");
            return result.toString();
        } else if (arr.length == 2) {
            result.append(arr[0]).append(" ");
            result.append(arr[1].substring(0, 1)).append(". ");
            return result.toString();
        } else {
            return fullAuthorName;
        }
    }

    public static void gridViewHeightWrap(GridView gridView) {
        ViewGroup.LayoutParams layoutParams = gridView.getLayoutParams();
        layoutParams.height = ViewGroup.LayoutParams.WRAP_CONTENT;
        gridView.setLayoutParams(layoutParams);
    }

    public static void gridViewHeight(GridView gridView, Activity activity) {
        ViewGroup.LayoutParams layoutParams = gridView.getLayoutParams();
        layoutParams.height = convertDpToPixel(37, activity);
        gridView.setLayoutParams(layoutParams);
    }

    public static void openFileFromDevice(String filePath, Activity activity) {
        String mime = Utils.getMimeByFilename(filePath);
        Intent intent = new Intent();
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        intent.setAction(Intent.ACTION_VIEW);
        intent.setDataAndType(Uri.fromFile(new File(filePath)), mime);
        try {
            activity.startActivity(intent);
        } catch (ActivityNotFoundException e) {
            AppMsg.makeText(activity, activity.getString(R.string.unknown_file) + " " +
                    filePath.substring(filePath.lastIndexOf("/") + 1, filePath.length()), AppMsg.STYLE_ALERT).show();
        }
    }

    /**
     * Возвращает размер в пикселях, из dp
     *
     * @param id       идентификатор ресурса
     * @param activity экран, из которого запускается метод
     * @return размер в пикселях
     */
    public static int getPxFromDp(int id, Activity activity) {
        return (int) (activity.getResources().getDimension(id) / activity.getResources().getDisplayMetrics().scaledDensity);
    }

    /**
     * Возвращает размер в пикселях, из dp
     *
     * @param id      идентификатор ресурса
     * @param context контекст приложения
     * @return размер в пикселях
     */
    public static int getPxFromDp(int id, Context context) {
        return (int) (context.getResources().getDimension(id) / context.getResources().getDisplayMetrics().scaledDensity);
    }

    /**
     * Метод пытается распарсить строку в double
     *
     * @param obj строка
     * @return значение
     */
    public static Double getDoubleStat(Object obj) {
        if (obj == null) {
            return 0d;
        }
        try {
            return Double.valueOf(obj.toString());
        } catch (Exception e) {
            try {
                return Double.parseDouble(obj.toString());
            } catch (Exception exc) {
                return 0d;
            }
        }
    }

    /**
     * Метод возвращает имя процесса по его индентификатору
     *
     * @param person пользователь
     * @param typeID идентификатор процесса
     * @return имя процесса
     */
    public static String getNameByTypeID(Person person, String typeID) {
        if (person == null || person.getProcesses() == null || typeID == null) {
            return "";
        }
        for (ProcessItem process : person.getProcesses()) {
            if (process.getType().equals(typeID)) {
                return process.getName();
            }
        }
        return "";
    }

    /**
     * Метод возвращает список обычных процессов (не спецпроцессов)
     *
     * @return список процессов
     */
    public static List<ProcessItem> getProcesses(Person person) {
        List<ProcessItem> items = new ArrayList<>();
        for (ProcessItem item : person.getProcesses()) {
            if (!item.getNumber().equals("")) {
                items.add(item);
            }
        }
        return items;
    }

    /**
     * Метод возвращает наименование типа длительности, исходя из типа
     *
     * @param type тип
     * @return наименование
     * @see RoutesConst#LENGTH_DAYS
     * @see RoutesConst#LENGTH_HOURS
     * @see RoutesConst#LENGTH_DATE
     */
    public static String getNameByLengthType(String type, Activity activity) {
        switch (type) {
            case RoutesConst.LENGTH_HOURS:
                return activity.getResources().getString(R.string.length_hours);
            case RoutesConst.LENGTH_DAYS:
                return activity.getResources().getString(R.string.length_days);
            case RoutesConst.LENGTH_DATE:
                return activity.getResources().getString(R.string.length_date);
        }
        return "";
    }

    /**
     * Конвертация из dp в px
     *
     * @param dp       dp
     * @param activity activity
     * @return px
     */
    public static int dpToPx(int dp, Activity activity) {
        DisplayMetrics displayMetrics = activity.getResources().getDisplayMetrics();
        return Math.round(dp * (displayMetrics.xdpi / DisplayMetrics.DENSITY_DEFAULT));
    }

    /**
     * Метод возщвращает новое имя файла добавленного с камеры
     *
     * @param photoFileName имя файла в миллисекундах
     * @param context       контекст приложения
     * @return имя файла
     */
    public static String getNewPhotoName(long photoFileName, Context context) {
        return context.getString(R.string.photo_name) + " " + DateUtils.getNewFileFormat(photoFileName) + Const.DEFAULT_FILE_FORMAT;
    }
}